#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <linux/sched.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>

#define SWITCH_NETNS 1
#define SIM_PORT 10000

pid_t childPid;

void cleanup(void) {
    printf("cleaning up...");
    if (childPid) kill(childPid, SIGKILL);

    system("ip link del sim_server");
}

int sendFunc(void) {
    int fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd == -1) {
	perror("server socket");
	exit(1);
    }

    struct sockaddr_in saddr;
    memset(&saddr, 0, sizeof(saddr));
    saddr.sin_family = AF_INET;
    saddr.sin_port = htons(SIM_PORT);
    inet_aton("9.9.9.1", &saddr.sin_addr);
    if (bind(fd, (struct sockaddr *) &saddr, sizeof(saddr)) < 0) {
	perror("server bind");
	return 1;
    }

    memset(&saddr, 0, sizeof(saddr));
    saddr.sin_family = AF_INET;
    saddr.sin_port = htons(SIM_PORT);
    inet_aton("9.9.9.2", &saddr.sin_addr);

    char buffer[] = "hello world 0123456789";
    printf("sending UPD packet\n");
    int ret = sendto(fd, buffer, sizeof(buffer), 0, (struct sockaddr *) &saddr, sizeof(saddr));
    if (ret < 0) {
	perror("send");
    } else if ((unsigned) ret < sizeof(buffer)) {
	printf("not all data sent\n");
    } else {
	printf("send OK\n");
    }
    return 0;
}

int main() {
    atexit(cleanup);

    system("ip link add name sim_client type veth peer name sim_server");
    system("ip addr add 9.9.9.1/24 dev sim_server");
    system("ip link set sim_server up");

    // create client
    if (!(childPid = fork())) {
#if SWITCH_NETNS
	if (unshare(CLONE_NEWNET)) {
	    perror("parent: unshare");
	    exit(-1);
	}
#endif
	execl("./child", "./child", NULL);
	exit(-1);
    }
#if SWITCH_NETNS
    char command[100];
    sprintf(command, "ip link set sim_client netns %d", childPid);
    system(command);
#endif
    sleep(2);

    sendFunc();

    sleep(5);
    return 0;
}
